<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;
use Tymon\JWTAuth\Contracts\JWTSubject;

class User extends Authenticatable implements JWTSubject
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var string[]
     */
    protected $fillable = [
        'name',
        'email',
        'password',
        'mobile',
        'city_id',
        'category_id',
        'subscription_id',
        'type',
        'verification_status',
        'verification_code',
        'status',
        'profile_image',
        'end_subscription',
        'admin_approve'
    ];
    protected $appends = ['in_subscribe'];

    public function getInSubscribeAttribute()
    {


        $check = Transfer::whereUserId($this->id)->whereIn('status', ['new', 'accepted'])->count();

        if ($check > 0) {
            $in_subscribe = true;
        } else {
            $in_subscribe = false;
        }


        return $in_subscribe;

    }

    public function city()
    {
        return $this->belongsTo(City::class, 'city_id');
    }

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    public function getJWTIdentifier()
    {
        return $this->getKey();
    }

    public function getJWTCustomClaims()
    {
        return [];
    }

    public function favourites()
    {
        return $this->belongsToMany(Service::class, 'favourites', 'user_id', 'service_id');
    }


    public function category()
    {
        return $this->belongsTo(Category::class, 'category_id');
    }

    public function subscription()
    {
        return $this->belongsTo(Package::class, 'subscription_id');
    }


}
